"""
Tests for RawDataReader instrument readers.

Each instrument has its own test module that tests:
1. Basic file reading
2. Different output formats/modes
3. Edge cases (duplicate timestamps, multiple headers, etc.)
4. QC flag generation
"""
